#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import univ

from pyasn1_modules import pem, rfc2985, rfc5280, rfc5652, rfc7292


class PKCS9AttrsTestCase(unittest.TestCase):
    pem_text = """\

HktMK+isIjxOTk4yJTOOAgIH0A==
"""

    def setUp(self):
        self.asn1Spec = rfc2985.AttributeSet()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(der_encoder(asn1Object), substrate)

        openTypesMap = {
            rfc2985.pkcs_9_at_smimeCapabilities: rfc2985.SMIMECapabilities(),
        }
        openTypesMap.update(rfc5280.certificateAttributesMap)
        openTypesMap.update(rfc5652.cmsAttributesMap)

        for attr in asn1Object:
            self.assertIn(attr["type"], openTypesMap)

            av, rest = der_decoder(
                attr["values"][0], asn1Spec=openTypesMap[attr["type"]]
            )

            self.assertFalse(rest)
            self.assertTrue(av.prettyPrint())
            self.assertEqual(attr["values"][0], der_encoder(av))

            if attr["type"] == rfc2985.pkcs_9_at_userPKCS12:

                self.assertEqual(univ.Integer(3), av["version"])
                self.assertEqual(rfc5652.id_data, av["authSafe"]["contentType"])

                outdata, rest = der_decoder(
                    av["authSafe"]["content"], asn1Spec=univ.OctetString()
                )

                self.assertFalse(rest)

                authsafe, rest = der_decoder(
                    outdata, asn1Spec=rfc7292.AuthenticatedSafe()
                )

                self.assertFalse(rest)

                for ci in authsafe:
                    self.assertEqual(rfc5652.id_data, ci["contentType"])

                    indata, rest = der_decoder(
                        ci["content"], asn1Spec=univ.OctetString()
                    )

                    self.assertFalse(rest)

                    sc, rest = der_decoder(indata, asn1Spec=rfc7292.SafeContents())

                    self.assertFalse(rest)

                    for sb in sc:
                        if sb["bagId"] in rfc7292.pkcs12BagTypeMap:
                            bv, rest = der_decoder(
                                sb["bagValue"],
                                asn1Spec=rfc7292.pkcs12BagTypeMap[sb["bagId"]],
                            )

                            self.assertFalse(rest)

                            for bagattr in sb["bagAttributes"]:
                                if bagattr["attrType"] in openTypesMap:
                                    inav, rest = der_decoder(
                                        bagattr["attrValues"][0],
                                        asn1Spec=openTypesMap[bagattr["attrType"]],
                                    )

                                    self.assertFalse(rest)

                                    if (
                                        bagattr["attrType"]
                                        == rfc2985.pkcs_9_at_friendlyName
                                    ):
                                        self.assertEqual(
                                            "3f71af65-1687-444a-9f46-c8be194c3e8e", inav
                                        )

                                    if (
                                        bagattr["attrType"]
                                        == rfc2985.pkcs_9_at_localKeyId
                                    ):
                                        self.assertEqual(
                                            univ.OctetString(hexValue="01000000"), inav
                                        )

            if attr["type"] == rfc2985.pkcs_9_at_pkcs7PDU:
                ci, rest = der_decoder(
                    attr["values"][0], asn1Spec=rfc5652.ContentInfo()
                )

                self.assertFalse(rest)
                self.assertEqual(rfc5652.id_signedData, ci["contentType"])

                sd, rest = der_decoder(ci["content"], asn1Spec=rfc5652.SignedData())

                self.assertFalse(rest)
                self.assertEqual(1, sd["version"])

                for si in sd["signerInfos"]:
                    self.assertEqual(1, si["version"])

                    for siattr in si["signedAttrs"]:
                        if siattr["attrType"] in openTypesMap:
                            siav, rest = der_decoder(
                                siattr["attrValues"][0],
                                asn1Spec=openTypesMap[siattr["attrType"]],
                            )

                            self.assertFalse(rest)

                            if siattr["attrType"] == rfc2985.pkcs_9_at_contentType:
                                self.assertEqual(rfc5652.id_data, siav)

                            if siattr["attrType"] == rfc2985.pkcs_9_at_messageDigest:
                                self.assertEqual("b6e422a4", siav.prettyPrint()[2:10])

                            if siattr["attrType"] == rfc2985.pkcs_9_at_signingTime:
                                self.assertEqual("190529182319Z", siav["utcTime"])

                for choices in sd["certificates"]:
                    for rdn in choices[0]["tbsCertificate"]["subject"]["rdnSequence"]:
                        if rdn[0]["type"] in openTypesMap:
                            nv, rest = der_decoder(
                                rdn[0]["value"], asn1Spec=openTypesMap[rdn[0]["type"]]
                            )
                            self.assertFalse(rest)

                            if rdn[0]["type"] == rfc2985.pkcs_9_at_emailAddress:
                                self.assertEqual("alice@example.com", nv)

    def testOpenTypes(self):
        openTypesMap = {
            rfc2985.pkcs_9_at_smimeCapabilities: rfc2985.SMIMECapabilities(),
        }
        openTypesMap.update(rfc5280.certificateAttributesMap)
        openTypesMap.update(rfc5652.cmsAttributesMap)

        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate,
            asn1Spec=self.asn1Spec,
            openTypes=openTypesMap,
            decodeOpenTypes=True,
        )

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        for attr in asn1Object:
            self.assertTrue(attr["type"], openTypesMap)

            if attr["type"] == rfc2985.pkcs_9_at_userPKCS12:

                self.assertEqual(univ.Integer(3), attr["values"][0]["version"])
                self.assertEqual(
                    rfc5652.id_data, attr["values"][0]["authSafe"]["contentType"]
                )

                authsafe, rest = der_decoder(
                    attr["values"][0]["authSafe"]["content"],
                    asn1Spec=rfc7292.AuthenticatedSafe(),
                )

                self.assertFalse(rest)

                for ci in authsafe:
                    self.assertEqual(rfc5652.id_data, ci["contentType"])

                    indata, rest = der_decoder(
                        ci["content"], asn1Spec=univ.OctetString()
                    )

                    self.assertFalse(rest)

                    sc, rest = der_decoder(
                        indata, asn1Spec=rfc7292.SafeContents(), decodeOpenTypes=True
                    )

                    self.assertFalse(rest)

                    for sb in sc:
                        if sb["bagId"] in rfc7292.pkcs12BagTypeMap:
                            for bagattr in sb["bagAttributes"]:
                                if bagattr["attrType"] in openTypesMap:

                                    if (
                                        bagattr["attrType"]
                                        == rfc2985.pkcs_9_at_friendlyName
                                    ):
                                        self.assertEqual(
                                            "3f71af65-1687-444a-9f46-c8be194c3e8e",
                                            bagattr["attrValues"][0],
                                        )

                                    if (
                                        bagattr["attrType"]
                                        == rfc2985.pkcs_9_at_localKeyId
                                    ):
                                        self.assertEqual(
                                            univ.OctetString(hexValue="01000000"),
                                            bagattr["attrValues"][0],
                                        )

            if attr["type"] == rfc2985.pkcs_9_at_pkcs7PDU:
                self.assertEqual(
                    rfc5652.id_signedData, attr["values"][0]["contentType"]
                )
                self.assertEqual(1, attr["values"][0]["content"]["version"])

                for si in attr["values"][0]["content"]["signerInfos"]:
                    self.assertEqual(1, si["version"])

                    for siattr in si["signedAttrs"]:
                        if siattr["attrType"] in openTypesMap:

                            if siattr["attrType"] == rfc2985.pkcs_9_at_contentType:
                                self.assertEqual(
                                    rfc5652.id_data, siattr["attrValues"][0]
                                )

                            if siattr["attrType"] == rfc2985.pkcs_9_at_messageDigest:
                                self.assertEqual(
                                    "b6e422a4",
                                    siattr["attrValues"][0].prettyPrint()[2:10],
                                )

                            if siattr["attrType"] == rfc2985.pkcs_9_at_signingTime:
                                self.assertEqual(
                                    "190529182319Z", siattr["attrValues"][0]["utcTime"]
                                )

                for choices in attr["values"][0]["content"]["certificates"]:
                    for rdn in choices[0]["tbsCertificate"]["subject"]["rdnSequence"]:
                        if rdn[0]["type"] in openTypesMap:
                            if rdn[0]["type"] == rfc2985.pkcs_9_at_emailAddress:
                                self.assertEqual("alice@example.com", rdn[0]["value"])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# OCSP request/response syntax
#
# Derived from a minimal OCSP library (RFC2560) code written by
# Bud P. Bruegger <bud@ancitel.it>
# Copyright: Ancitel, S.p.a,  Rome, Italy
# License: BSD
#

#
# current limitations:
# * request and response works only for a single certificate
# * only some values are parsed out of the response
# * the request does't set a nonce nor signature
# * there is no signature validation of the response
# * dates are left as strings in GeneralizedTime format -- datetime.datetime
# would be nicer
#
from pyasn1.type import namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc2459

# Start of OCSP module definitions

# This should be in directory Authentication Framework (X.509) module


class CRLReason(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ("unspecified", 0),
        ("keyCompromise", 1),
        ("cACompromise", 2),
        ("affiliationChanged", 3),
        ("superseded", 4),
        ("cessationOfOperation", 5),
        ("certificateHold", 6),
        ("removeFromCRL", 8),
        ("privilegeWithdrawn", 9),
        ("aACompromise", 10),
    )


# end of directory Authentication Framework (X.509) module

# This should be in PKIX Certificate Extensions module


class GeneralName(univ.OctetString):
    pass


# end of PKIX Certificate Extensions module

id_kp_OCSPSigning = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 3, 9))
id_pkix_ocsp = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1))
id_pkix_ocsp_basic = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1, 1))
id_pkix_ocsp_nonce = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1, 2))
id_pkix_ocsp_crl = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1, 3))
id_pkix_ocsp_response = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1, 4))
id_pkix_ocsp_nocheck = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1, 5))
id_pkix_ocsp_archive_cutoff = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1, 6))
id_pkix_ocsp_service_locator = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, 1, 7))


class AcceptableResponses(univ.SequenceOf):
    componentType = univ.ObjectIdentifier()


class ArchiveCutoff(useful.GeneralizedTime):
    pass


class UnknownInfo(univ.Null):
    pass


class RevokedInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("revocationTime", useful.GeneralizedTime()),
        namedtype.OptionalNamedType(
            "revocationReason",
            CRLReason().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class CertID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("hashAlgorithm", rfc2459.AlgorithmIdentifier()),
        namedtype.NamedType("issuerNameHash", univ.OctetString()),
        namedtype.NamedType("issuerKeyHash", univ.OctetString()),
        namedtype.NamedType("serialNumber", rfc2459.CertificateSerialNumber()),
    )


class CertStatus(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "good",
            univ.Null().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "revoked",
            RevokedInfo().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "unknown",
            UnknownInfo().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class SingleResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certID", CertID()),
        namedtype.NamedType("certStatus", CertStatus()),
        namedtype.NamedType("thisUpdate", useful.GeneralizedTime()),
        namedtype.OptionalNamedType(
            "nextUpdate",
            useful.GeneralizedTime().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "singleExtensions",
            rfc2459.Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class KeyHash(univ.OctetString):
    pass


class ResponderID(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "byName",
            rfc2459.Name().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "byKey",
            KeyHash().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class Version(univ.Integer):
    namedValues = namedval.NamedValues(("v1", 0))


class ResponseData(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "version",
            Version("v1").subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType("responderID", ResponderID()),
        namedtype.NamedType("producedAt", useful.GeneralizedTime()),
        namedtype.NamedType(
            "responses", univ.SequenceOf(componentType=SingleResponse())
        ),
        namedtype.OptionalNamedType(
            "responseExtensions",
            rfc2459.Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class BasicOCSPResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("tbsResponseData", ResponseData()),
        namedtype.NamedType("signatureAlgorithm", rfc2459.AlgorithmIdentifier()),
        namedtype.NamedType("signature", univ.BitString()),
        namedtype.OptionalNamedType(
            "certs",
            univ.SequenceOf(componentType=rfc2459.Certificate()).subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class ResponseBytes(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("responseType", univ.ObjectIdentifier()),
        namedtype.NamedType("response", univ.OctetString()),
    )


class OCSPResponseStatus(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ("successful", 0),
        ("malformedRequest", 1),
        ("internalError", 2),
        ("tryLater", 3),
        ("undefinedStatus", 4),  # should never occur
        ("sigRequired", 5),
        ("unauthorized", 6),
    )


class OCSPResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("responseStatus", OCSPResponseStatus()),
        namedtype.OptionalNamedType(
            "responseBytes",
            ResponseBytes().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class Request(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("reqCert", CertID()),
        namedtype.OptionalNamedType(
            "singleRequestExtensions",
            rfc2459.Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class Signature(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signatureAlgorithm", rfc2459.AlgorithmIdentifier()),
        namedtype.NamedType("signature", univ.BitString()),
        namedtype.OptionalNamedType(
            "certs",
            univ.SequenceOf(componentType=rfc2459.Certificate()).subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class TBSRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "version",
            Version("v1").subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "requestorName",
            GeneralName().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType("requestList", univ.SequenceOf(componentType=Request())),
        namedtype.OptionalNamedType(
            "requestExtensions",
            rfc2459.Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class OCSPRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("tbsRequest", TBSRequest()),
        namedtype.OptionalNamedType(
            "optionalSignature",
            Signature().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )



#!/usr/bin/env python
#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Read ASN.1/PEM X.509 certificate requests (PKCS#10 format) on stdin,
# parse each into plain text, then build substrate from it
#
import sys

from pyasn1.codec.der import decoder, encoder

from pyasn1_modules import pem, rfc2314

if len(sys.argv) != 1:
    print(
        """Usage:
$ cat certificateRequest.pem | %s"""
        % sys.argv[0]
    )
    sys.exit(-1)

certType = rfc2314.CertificationRequest()

certCnt = 0

while True:
    idx, substrate = pem.readPemBlocksFromFile(
        sys.stdin,
        ("-----BEGIN CERTIFICATE REQUEST-----", "-----END CERTIFICATE REQUEST-----"),
    )
    if not substrate:
        break

    cert, rest = decoder.decode(substrate, asn1Spec=certType)

    if rest:
        substrate = substrate[: -len(rest)]

    print(cert.prettyPrint())

    assert encoder.encode(cert) == substrate, "cert recode fails"

    certCnt += 1

print("*** %s PEM certificate request(s) de/serialized" % certCnt)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Certificate Store Access via HTTP
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4387.txt
#


from pyasn1.type import univ

id_ad = univ.ObjectIdentifier(
    (
        1,
        3,
        6,
        1,
        5,
        5,
        7,
        48,
    )
)

id_ad_http_certs = id_ad + (6,)

id_ad_http_crls = id_ad + (7,)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# CMS Advanced Electronic Signatures (CAdES)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5126.txt
#

from pyasn1.type import char, constraint, namedtype, opentype, tag, univ, useful

from pyasn1_modules import rfc3161, rfc5035, rfc5280, rfc5652, rfc5755, rfc6960

MAX = float("inf")


# Maps for OpenTypes

commitmentQualifierMap = {}

sigQualifiersMap = {}

otherRevRefMap = {}

otherRevValMap = {}


# Imports from RFC 5652

ContentInfo = rfc5652.ContentInfo

ContentType = rfc5652.ContentType

SignedData = rfc5652.SignedData

EncapsulatedContentInfo = rfc5652.EncapsulatedContentInfo

SignerInfo = rfc5652.SignerInfo

MessageDigest = rfc5652.MessageDigest

SigningTime = rfc5652.SigningTime

Countersignature = rfc5652.Countersignature

id_data = rfc5652.id_data

id_signedData = rfc5652.id_signedData

id_contentType = rfc5652.id_contentType

id_messageDigest = rfc5652.id_messageDigest

id_signingTime = rfc5652.id_signingTime

id_countersignature = rfc5652.id_countersignature


# Imports from RFC 5035

SigningCertificate = rfc5035.SigningCertificate

IssuerSerial = rfc5035.IssuerSerial

ContentReference = rfc5035.ContentReference

ContentIdentifier = rfc5035.ContentIdentifier

id_aa_contentReference = rfc5035.id_aa_contentReference

id_aa_contentIdentifier = rfc5035.id_aa_contentIdentifier

id_aa_signingCertificate = rfc5035.id_aa_signingCertificate

id_aa_signingCertificateV2 = rfc5035.id_aa_signingCertificateV2


# Imports from RFC 5280

Certificate = rfc5280.Certificate

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

CertificateList = rfc5280.CertificateList

Name = rfc5280.Name

Attribute = rfc5280.Attribute

GeneralNames = rfc5280.GeneralNames

GeneralName = rfc5280.GeneralName

PolicyInformation = rfc5280.PolicyInformation

DirectoryString = rfc5280.DirectoryString


# Imports from RFC 5755

AttributeCertificate = rfc5755.AttributeCertificate


# Imports from RFC 6960

BasicOCSPResponse = rfc6960.BasicOCSPResponse

ResponderID = rfc6960.ResponderID


# Imports from RFC 3161

TimeStampToken = rfc3161.TimeStampToken


# OID used referencing electronic signature mechanisms

id_etsi_es_IDUP_Mechanism_v1 = univ.ObjectIdentifier("0.4.0.1733.1.4.1")


# OtherSigningCertificate - deprecated

id_aa_ets_otherSigCert = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.19")


class OtherHashValue(univ.OctetString):
    pass


class OtherHashAlgAndValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("hashAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("hashValue", OtherHashValue()),
    )


class OtherHash(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("sha1Hash", OtherHashValue()),
        namedtype.NamedType("otherHash", OtherHashAlgAndValue()),
    )


class OtherCertID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("otherCertHash", OtherHash()),
        namedtype.OptionalNamedType("issuerSerial", IssuerSerial()),
    )


class OtherSigningCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certs", univ.SequenceOf(componentType=OtherCertID())),
        namedtype.OptionalNamedType(
            "policies", univ.SequenceOf(componentType=PolicyInformation())
        ),
    )


# Signature Policy Identifier

id_aa_ets_sigPolicyId = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.15")


class SigPolicyId(univ.ObjectIdentifier):
    pass


class SigPolicyHash(OtherHashAlgAndValue):
    pass


class SigPolicyQualifierId(univ.ObjectIdentifier):
    pass


class SigPolicyQualifierInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("sigPolicyQualifierId", SigPolicyQualifierId()),
        namedtype.NamedType(
            "sigQualifier",
            univ.Any(),
            openType=opentype.OpenType("sigPolicyQualifierId", sigQualifiersMap),
        ),
    )


class SignaturePolicyId(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("sigPolicyId", SigPolicyId()),
        namedtype.NamedType("sigPolicyHash", SigPolicyHash()),
        namedtype.OptionalNamedType(
            "sigPolicyQualifiers",
            univ.SequenceOf(componentType=SigPolicyQualifierInfo()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


class SignaturePolicyImplied(univ.Null):
    pass


class SignaturePolicy(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("signaturePolicyId", SignaturePolicyId()),
        namedtype.NamedType("signaturePolicyImplied", SignaturePolicyImplied()),
    )


id_spq_ets_unotice = univ.ObjectIdentifier("1.2.840.113549.1.9.16.5.2")


class DisplayText(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "visibleString",
            char.VisibleString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200)
            ),
        ),
    )


class NoticeReference(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("organization", DisplayText()),
        namedtype.NamedType(
            "noticeNumbers", univ.SequenceOf(componentType=univ.Integer())
        ),
    )


class SPUserNotice(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType("noticeRef", NoticeReference()),
        namedtype.OptionalNamedType("explicitText", DisplayText()),
    )


noticeToUser = SigPolicyQualifierInfo()
noticeToUser["sigPolicyQualifierId"] = id_spq_ets_unotice
noticeToUser["sigQualifier"] = SPUserNotice()


id_spq_ets_uri = univ.ObjectIdentifier("1.2.840.113549.1.9.16.5.1")


class SPuri(char.IA5String):
    pass


pointerToSigPolSpec = SigPolicyQualifierInfo()
pointerToSigPolSpec["sigPolicyQualifierId"] = id_spq_ets_uri
pointerToSigPolSpec["sigQualifier"] = SPuri()


# Commitment Type

id_aa_ets_commitmentType = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.16")


class CommitmentTypeIdentifier(univ.ObjectIdentifier):
    pass


class CommitmentTypeQualifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("commitmentTypeIdentifier", CommitmentTypeIdentifier()),
        namedtype.NamedType(
            "qualifier",
            univ.Any(),
            openType=opentype.OpenType(
                "commitmentTypeIdentifier", commitmentQualifierMap
            ),
        ),
    )


class CommitmentTypeIndication(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("commitmentTypeId", CommitmentTypeIdentifier()),
        namedtype.OptionalNamedType(
            "commitmentTypeQualifier",
            univ.SequenceOf(componentType=CommitmentTypeQualifier()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


id_cti_ets_proofOfOrigin = univ.ObjectIdentifier("1.2.840.113549.1.9.16.6.1")

id_cti_ets_proofOfReceipt = univ.ObjectIdentifier("1.2.840.113549.1.9.16.6.2")

id_cti_ets_proofOfDelivery = univ.ObjectIdentifier("1.2.840.113549.1.9.16.6.3")

id_cti_ets_proofOfSender = univ.ObjectIdentifier("1.2.840.113549.1.9.16.6.4")

id_cti_ets_proofOfApproval = univ.ObjectIdentifier("1.2.840.113549.1.9.16.6.5")

id_cti_ets_proofOfCreation = univ.ObjectIdentifier("1.2.840.113549.1.9.16.6.6")


# Signer Location

id_aa_ets_signerLocation = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.17")


class PostalAddress(univ.SequenceOf):
    componentType = DirectoryString()
    subtypeSpec = constraint.ValueSizeConstraint(1, 6)


class SignerLocation(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "countryName",
            DirectoryString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "localityName",
            DirectoryString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "postalAdddress",
            PostalAddress().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


# Signature Timestamp

id_aa_signatureTimeStampToken = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.14")


class SignatureTimeStampToken(TimeStampToken):
    pass


# Content Timestamp

id_aa_ets_contentTimestamp = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.20")


class ContentTimestamp(TimeStampToken):
    pass


# Signer Attributes

id_aa_ets_signerAttr = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.18")


class ClaimedAttributes(univ.SequenceOf):
    componentType = Attribute()


class CertifiedAttributes(AttributeCertificate):
    pass


class SignerAttribute(univ.SequenceOf):
    componentType = univ.Choice(
        componentType=namedtype.NamedTypes(
            namedtype.NamedType(
                "claimedAttributes",
                ClaimedAttributes().subtype(
                    explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
                ),
            ),
            namedtype.NamedType(
                "certifiedAttributes",
                CertifiedAttributes().subtype(
                    explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
                ),
            ),
        )
    )


# Complete Certificate Refs

id_aa_ets_certificateRefs = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.21")


class CompleteCertificateRefs(univ.SequenceOf):
    componentType = OtherCertID()


# Complete Revocation Refs

id_aa_ets_revocationRefs = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.22")


class CrlIdentifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("crlissuer", Name()),
        namedtype.NamedType("crlIssuedTime", useful.UTCTime()),
        namedtype.OptionalNamedType("crlNumber", univ.Integer()),
    )


class CrlValidatedID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("crlHash", OtherHash()),
        namedtype.OptionalNamedType("crlIdentifier", CrlIdentifier()),
    )


class CRLListID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("crls", univ.SequenceOf(componentType=CrlValidatedID()))
    )


class OcspIdentifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("ocspResponderID", ResponderID()),
        namedtype.NamedType("producedAt", useful.GeneralizedTime()),
    )


class OcspResponsesID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("ocspIdentifier", OcspIdentifier()),
        namedtype.OptionalNamedType("ocspRepHash", OtherHash()),
    )


class OcspListID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "ocspResponses", univ.SequenceOf(componentType=OcspResponsesID())
        )
    )


class OtherRevRefType(univ.ObjectIdentifier):
    pass


class OtherRevRefs(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("otherRevRefType", OtherRevRefType()),
        namedtype.NamedType(
            "otherRevRefs",
            univ.Any(),
            openType=opentype.OpenType("otherRevRefType", otherRevRefMap),
        ),
    )


class CrlOcspRef(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "crlids",
            CRLListID().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "ocspids",
            OcspListID().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "otherRev",
            OtherRevRefs().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
    )


class CompleteRevocationRefs(univ.SequenceOf):
    componentType = CrlOcspRef()


# Certificate Values

id_aa_ets_certValues = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.23")


class CertificateValues(univ.SequenceOf):
    componentType = Certificate()


# Certificate Revocation Values

id_aa_ets_revocationValues = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.24")


class OtherRevValType(univ.ObjectIdentifier):
    pass


class OtherRevVals(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("otherRevValType", OtherRevValType()),
        namedtype.NamedType(
            "otherRevVals",
            univ.Any(),
            openType=opentype.OpenType("otherRevValType", otherRevValMap),
        ),
    )


class RevocationValues(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "crlVals",
            univ.SequenceOf(componentType=CertificateList()).subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "ocspVals",
            univ.SequenceOf(componentType=BasicOCSPResponse()).subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "otherRevVals",
            OtherRevVals().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
    )


# CAdES-C Timestamp

id_aa_ets_escTimeStamp = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.25")


class ESCTimeStampToken(TimeStampToken):
    pass


# Time-Stamped Certificates and CRLs

id_aa_ets_certCRLTimestamp = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.26")


class TimestampedCertsCRLs(TimeStampToken):
    pass


# Archive Timestamp

id_aa_ets_archiveTimestampV2 = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.48")


class ArchiveTimeStampToken(TimeStampToken):
    pass


# Attribute certificate references

id_aa_ets_attrCertificateRefs = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.44")


class AttributeCertificateRefs(univ.SequenceOf):
    componentType = OtherCertID()


# Attribute revocation references

id_aa_ets_attrRevocationRefs = univ.ObjectIdentifier("1.2.840.113549.1.9.16.2.45")


class AttributeRevocationRefs(univ.SequenceOf):
    componentType = CrlOcspRef()


# Update the sigQualifiersMap

_sigQualifiersMapUpdate = {
    id_spq_ets_unotice: SPUserNotice(),
    id_spq_ets_uri: SPuri(),
}

sigQualifiersMap.update(_sigQualifiersMapUpdate)


# Update the CMS Attribute Map in rfc5652.py

_cmsAttributesMapUpdate = {
    id_aa_ets_otherSigCert: OtherSigningCertificate(),
    id_aa_ets_sigPolicyId: SignaturePolicy(),
    id_aa_ets_commitmentType: CommitmentTypeIndication(),
    id_aa_ets_signerLocation: SignerLocation(),
    id_aa_signatureTimeStampToken: SignatureTimeStampToken(),
    id_aa_ets_contentTimestamp: ContentTimestamp(),
    id_aa_ets_signerAttr: SignerAttribute(),
    id_aa_ets_certificateRefs: CompleteCertificateRefs(),
    id_aa_ets_revocationRefs: CompleteRevocationRefs(),
    id_aa_ets_certValues: CertificateValues(),
    id_aa_ets_revocationValues: RevocationValues(),
    id_aa_ets_escTimeStamp: ESCTimeStampToken(),
    id_aa_ets_certCRLTimestamp: TimestampedCertsCRLs(),
    id_aa_ets_archiveTimestampV2: ArchiveTimeStampToken(),
    id_aa_ets_attrCertificateRefs: AttributeCertificateRefs(),
    id_aa_ets_attrRevocationRefs: AttributeRevocationRefs(),
}

rfc5652.cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with some assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Using Pre-Shared Key (PSK) in the Cryptographic Message Syntax (CMS)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8696.txt
#

from pyasn1.type import constraint, namedtype, namedval, tag, univ

from pyasn1_modules import rfc5652

MAX = float("inf")


id_ori = univ.ObjectIdentifier("1.2.840.113549.1.9.16.13")

id_ori_keyTransPSK = univ.ObjectIdentifier("1.2.840.113549.1.9.16.13.1")

id_ori_keyAgreePSK = univ.ObjectIdentifier("1.2.840.113549.1.9.16.13.2")


class PreSharedKeyIdentifier(univ.OctetString):
    pass


class KeyTransRecipientInfos(univ.SequenceOf):
    componentType = rfc5652.KeyTransRecipientInfo()


class KeyTransPSKRecipientInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", rfc5652.CMSVersion()),
        namedtype.NamedType("pskid", PreSharedKeyIdentifier()),
        namedtype.NamedType("kdfAlgorithm", rfc5652.KeyDerivationAlgorithmIdentifier()),
        namedtype.NamedType(
            "keyEncryptionAlgorithm", rfc5652.KeyEncryptionAlgorithmIdentifier()
        ),
        namedtype.NamedType("ktris", KeyTransRecipientInfos()),
        namedtype.NamedType("encryptedKey", rfc5652.EncryptedKey()),
    )


class KeyAgreePSKRecipientInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", rfc5652.CMSVersion()),
        namedtype.NamedType("pskid", PreSharedKeyIdentifier()),
        namedtype.NamedType(
            "originator",
            rfc5652.OriginatorIdentifierOrKey().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "ukm",
            rfc5652.UserKeyingMaterial().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType("kdfAlgorithm", rfc5652.KeyDerivationAlgorithmIdentifier()),
        namedtype.NamedType(
            "keyEncryptionAlgorithm", rfc5652.KeyEncryptionAlgorithmIdentifier()
        ),
        namedtype.NamedType("recipientEncryptedKeys", rfc5652.RecipientEncryptedKeys()),
    )


class CMSORIforPSKOtherInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("psk", univ.OctetString()),
        namedtype.NamedType(
            "keyMgmtAlgType",
            univ.Enumerated(
                namedValues=namedval.NamedValues(("keyTrans", 5), ("keyAgree", 10))
            ),
        ),
        namedtype.NamedType(
            "keyEncryptionAlgorithm", rfc5652.KeyEncryptionAlgorithmIdentifier()
        ),
        namedtype.NamedType(
            "pskLength",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(1, MAX)),
        ),
        namedtype.NamedType(
            "kdkLength",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(1, MAX)),
        ),
    )


# Update the CMS Other Recipient Info map in rfc5652.py

_otherRecipientInfoMapUpdate = {
    id_ori_keyTransPSK: KeyTransPSKRecipientInfo(),
    id_ori_keyAgreePSK: KeyAgreePSKRecipientInfo(),
}

rfc5652.otherRecipientInfoMap.update(_otherRecipientInfoMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Using the GOST 28147-89, GOST R 34.11-94, GOST R 34.10-94, and
#   GOST R 34.10-2001 Algorithms with the CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4490.txt
#


from pyasn1.type import char, constraint, namedtype, namedval, tag, univ, useful

from pyasn1_modules import rfc4357, rfc5280

# Imports from RFC 4357

id_CryptoPro_algorithms = rfc4357.id_CryptoPro_algorithms

id_GostR3410_94 = rfc4357.id_GostR3410_94

id_GostR3410_2001 = rfc4357.id_GostR3410_2001

Gost28147_89_ParamSet = rfc4357.Gost28147_89_ParamSet

Gost28147_89_EncryptedKey = rfc4357.Gost28147_89_EncryptedKey

GostR3410_94_PublicKeyParameters = rfc4357.GostR3410_94_PublicKeyParameters

GostR3410_2001_PublicKeyParameters = rfc4357.GostR3410_2001_PublicKeyParameters


# Imports from RFC 5280

SubjectPublicKeyInfo = rfc5280.SubjectPublicKeyInfo


# CMS/PKCS#7 key agreement algorithms & parameters


class Gost28147_89_KeyWrapParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("encryptionParamSet", Gost28147_89_ParamSet()),
        namedtype.OptionalNamedType(
            "ukm",
            univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(8, 8)
            ),
        ),
    )


id_Gost28147_89_CryptoPro_KeyWrap = id_CryptoPro_algorithms + (
    13,
    1,
)


id_Gost28147_89_None_KeyWrap = id_CryptoPro_algorithms + (
    13,
    0,
)


id_GostR3410_2001_CryptoPro_ESDH = id_CryptoPro_algorithms + (96,)


id_GostR3410_94_CryptoPro_ESDH = id_CryptoPro_algorithms + (97,)


# CMS/PKCS#7 key transport algorithms & parameters

id_GostR3410_2001_KeyTransportSMIMECapability = id_GostR3410_2001


id_GostR3410_94_KeyTransportSMIMECapability = id_GostR3410_94


class GostR3410_TransportParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("encryptionParamSet", Gost28147_89_ParamSet()),
        namedtype.OptionalNamedType(
            "ephemeralPublicKey",
            SubjectPublicKeyInfo().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "ukm",
            univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(8, 8)
            ),
        ),
    )


class GostR3410_KeyTransport(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("sessionEncryptedKey", Gost28147_89_EncryptedKey()),
        namedtype.OptionalNamedType(
            "transportParameters",
            GostR3410_TransportParameters().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )


# GOST R 34.10-94 signature algorithm & parameters


class GostR3410_94_Signature(univ.OctetString):
    subtypeSpec = constraint.ValueSizeConstraint(64, 64)


# GOST R 34.10-2001 signature algorithms and parameters


class GostR3410_2001_Signature(univ.OctetString):
    subtypeSpec = constraint.ValueSizeConstraint(64, 64)


# Update the Algorithm Identifier map in rfc5280.py

_algorithmIdentifierMapUpdate = {
    id_Gost28147_89_CryptoPro_KeyWrap: Gost28147_89_KeyWrapParameters(),
    id_Gost28147_89_None_KeyWrap: Gost28147_89_KeyWrapParameters(),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Updated by Russ Housley to resolve the TODO regarding the Certificate
#   Policies Certificate Extension.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# X.509 message syntax
#
# ASN.1 source from:
# http://www.trl.ibm.com/projects/xml/xss4j/data/asn1/grammars/x509.asn
# http://www.ietf.org/rfc/rfc2459.txt
#
# Sample captures from:
# http://wiki.wireshark.org/SampleCaptures/
#
from pyasn1.type import (
    char,
    constraint,
    namedtype,
    namedval,
    opentype,
    tag,
    univ,
    useful,
)

MAX = float("inf")

#
# PKIX1Explicit88
#

# Upper Bounds
ub_name = univ.Integer(32768)
ub_common_name = univ.Integer(64)
ub_locality_name = univ.Integer(128)
ub_state_name = univ.Integer(128)
ub_organization_name = univ.Integer(64)
ub_organizational_unit_name = univ.Integer(64)
ub_title = univ.Integer(64)
ub_match = univ.Integer(128)
ub_emailaddress_length = univ.Integer(128)
ub_common_name_length = univ.Integer(64)
ub_country_name_alpha_length = univ.Integer(2)
ub_country_name_numeric_length = univ.Integer(3)
ub_domain_defined_attributes = univ.Integer(4)
ub_domain_defined_attribute_type_length = univ.Integer(8)
ub_domain_defined_attribute_value_length = univ.Integer(128)
ub_domain_name_length = univ.Integer(16)
ub_extension_attributes = univ.Integer(256)
ub_e163_4_number_length = univ.Integer(15)
ub_e163_4_sub_address_length = univ.Integer(40)
ub_generation_qualifier_length = univ.Integer(3)
ub_given_name_length = univ.Integer(16)
ub_initials_length = univ.Integer(5)
ub_integer_options = univ.Integer(256)
ub_numeric_user_id_length = univ.Integer(32)
ub_organization_name_length = univ.Integer(64)
ub_organizational_unit_name_length = univ.Integer(32)
ub_organizational_units = univ.Integer(4)
ub_pds_name_length = univ.Integer(16)
ub_pds_parameter_length = univ.Integer(30)
ub_pds_physical_address_lines = univ.Integer(6)
ub_postal_code_length = univ.Integer(16)
ub_surname_length = univ.Integer(40)
ub_terminal_id_length = univ.Integer(24)
ub_unformatted_address_length = univ.Integer(180)
ub_x121_address_length = univ.Integer(16)


class UniversalString(char.UniversalString):
    pass


class BMPString(char.BMPString):
    pass


class UTF8String(char.UTF8String):
    pass


id_pkix = univ.ObjectIdentifier("1.3.6.1.5.5.7")
id_pe = univ.ObjectIdentifier("1.3.6.1.5.5.7.1")
id_qt = univ.ObjectIdentifier("1.3.6.1.5.5.7.2")
id_kp = univ.ObjectIdentifier("1.3.6.1.5.5.7.3")
id_ad = univ.ObjectIdentifier("1.3.6.1.5.5.7.48")

id_qt_cps = univ.ObjectIdentifier("1.3.6.1.5.5.7.2.1")
id_qt_unotice = univ.ObjectIdentifier("1.3.6.1.5.5.7.2.2")

id_ad_ocsp = univ.ObjectIdentifier("1.3.6.1.5.5.7.48.1")
id_ad_caIssuers = univ.ObjectIdentifier("1.3.6.1.5.5.7.48.2")


id_at = univ.ObjectIdentifier("2.5.4")
id_at_name = univ.ObjectIdentifier("2.5.4.41")
# preserve misspelled variable for compatibility
id_at_sutname = id_at_surname = univ.ObjectIdentifier("2.5.4.4")
id_at_givenName = univ.ObjectIdentifier("2.5.4.42")
id_at_initials = univ.ObjectIdentifier("2.5.4.43")
id_at_generationQualifier = univ.ObjectIdentifier("2.5.4.44")


class X520name(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)
            ),
        ),
    )


id_at_commonName = univ.ObjectIdentifier("2.5.4.3")


class X520CommonName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)
            ),
        ),
    )


id_at_localityName = univ.ObjectIdentifier("2.5.4.7")


class X520LocalityName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)
            ),
        ),
    )


id_at_stateOrProvinceName = univ.ObjectIdentifier("2.5.4.8")


class X520StateOrProvinceName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)
            ),
        ),
    )


id_at_organizationName = univ.ObjectIdentifier("2.5.4.10")


class X520OrganizationName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)
            ),
        ),
    )


id_at_organizationalUnitName = univ.ObjectIdentifier("2.5.4.11")


class X520OrganizationalUnitName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_organizational_unit_name
                )
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_organizational_unit_name
                )
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_organizational_unit_name
                )
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_organizational_unit_name
                )
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_organizational_unit_name
                )
            ),
        ),
    )


id_at_title = univ.ObjectIdentifier("2.5.4.12")


class X520Title(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)
            ),
        ),
    )


id_at_dnQualifier = univ.ObjectIdentifier("2.5.4.46")


class X520dnQualifier(char.PrintableString):
    pass


id_at_countryName = univ.ObjectIdentifier("2.5.4.6")


class X520countryName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(
        2, 2
    )


pkcs_9 = univ.ObjectIdentifier("1.2.840.113549.1.9")

emailAddress = univ.ObjectIdentifier("1.2.840.113549.1.9.1")


class Pkcs9email(char.IA5String):
    subtypeSpec = char.IA5String.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_emailaddress_length
    )


# ----


class DSAPrivateKey(univ.Sequence):
    """PKIX compliant DSA private key structure"""

    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "version", univ.Integer(namedValues=namedval.NamedValues(("v1", 0)))
        ),
        namedtype.NamedType("p", univ.Integer()),
        namedtype.NamedType("q", univ.Integer()),
        namedtype.NamedType("g", univ.Integer()),
        namedtype.NamedType("public", univ.Integer()),
        namedtype.NamedType("private", univ.Integer()),
    )


# ----


class DirectoryString(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "teletexString",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
        namedtype.NamedType(
            "printableString",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
        namedtype.NamedType(
            "universalString",
            char.UniversalString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
        namedtype.NamedType(
            "ia5String",
            char.IA5String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        )
        # hm, this should not be here!? XXX
    )


# certificate and CRL specific structures begin here


class AlgorithmIdentifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("algorithm", univ.ObjectIdentifier()),
        namedtype.OptionalNamedType("parameters", univ.Any()),
    )


# Algorithm OIDs and parameter structures

pkcs_1 = univ.ObjectIdentifier("1.2.840.113549.1.1")
rsaEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.1")
md2WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.2")
md5WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.4")
sha1WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.5")
id_dsa_with_sha1 = univ.ObjectIdentifier("1.2.840.10040.4.3")


class Dss_Sig_Value(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("r", univ.Integer()),
        namedtype.NamedType("s", univ.Integer()),
    )


dhpublicnumber = univ.ObjectIdentifier("1.2.840.10046.2.1")


class ValidationParms(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("seed", univ.BitString()),
        namedtype.NamedType("pgenCounter", univ.Integer()),
    )


class DomainParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("p", univ.Integer()),
        namedtype.NamedType("g", univ.Integer()),
        namedtype.NamedType("q", univ.Integer()),
        namedtype.NamedType("j", univ.Integer()),
        namedtype.OptionalNamedType("validationParms", ValidationParms()),
    )


id_dsa = univ.ObjectIdentifier("1.2.840.10040.4.1")


class Dss_Parms(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("p", univ.Integer()),
        namedtype.NamedType("q", univ.Integer()),
        namedtype.NamedType("g", univ.Integer()),
    )


# x400 address syntax starts here

teletex_domain_defined_attributes = univ.Integer(6)


class TeletexDomainDefinedAttribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "type",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_domain_defined_attribute_type_length
                )
            ),
        ),
        namedtype.NamedType("value", char.TeletexString()),
    )


class TeletexDomainDefinedAttributes(univ.SequenceOf):
    componentType = TeletexDomainDefinedAttribute()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(
        1, ub_domain_defined_attributes
    )


terminal_type = univ.Integer(23)


class TerminalType(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueSizeConstraint(
        0, ub_integer_options
    )
    namedValues = namedval.NamedValues(
        ("telex", 3),
        ("teletelex", 4),
        ("g3-facsimile", 5),
        ("g4-facsimile", 6),
        ("ia5-terminal", 7),
        ("videotex", 8),
    )


class PresentationAddress(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "pSelector",
            univ.OctetString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "sSelector",
            univ.OctetString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "tSelector",
            univ.OctetString().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "nAddresses",
            univ.SetOf(componentType=univ.OctetString()).subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3),
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX),
            ),
        ),
    )


extended_network_address = univ.Integer(22)


class E163_4_address(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "number",
            char.NumericString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_e163_4_number_length),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0),
            ),
        ),
        namedtype.OptionalNamedType(
            "sub-address",
            char.NumericString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_e163_4_sub_address_length
                ),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1),
            ),
        ),
    )


class ExtendedNetworkAddress(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("e163-4-address", E163_4_address()),
        namedtype.NamedType(
            "psap-address",
            PresentationAddress().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class PDSParameter(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "printable-string",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length)
            ),
        ),
        namedtype.OptionalNamedType(
            "teletex-string",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length)
            ),
        ),
    )


local_postal_attributes = univ.Integer(21)


class LocalPostalAttributes(PDSParameter):
    pass


class UniquePostalName(PDSParameter):
    pass


unique_postal_name = univ.Integer(20)

poste_restante_address = univ.Integer(19)


class PosteRestanteAddress(PDSParameter):
    pass


post_office_box_address = univ.Integer(18)


class PostOfficeBoxAddress(PDSParameter):
    pass


street_address = univ.Integer(17)


class StreetAddress(PDSParameter):
    pass


class UnformattedPostalAddress(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "printable-address",
            univ.SequenceOf(
                componentType=char.PrintableString()
                .subtype(
                    subtypeSpec=constraint.ValueSizeConstraint(
                        1, ub_pds_parameter_length
                    )
                )
                .subtype(
                    subtypeSpec=constraint.ValueSizeConstraint(
                        1, ub_pds_physical_address_lines
                    )
                )
            ),
        ),
        namedtype.OptionalNamedType(
            "teletex-string",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_unformatted_address_length
                )
            ),
        ),
    )


physical_delivery_office_name = univ.Integer(10)


class PhysicalDeliveryOfficeName(PDSParameter):
    pass


physical_delivery_office_number = univ.Integer(11)


class PhysicalDeliveryOfficeNumber(PDSParameter):
    pass


extension_OR_address_components = univ.Integer(12)


class ExtensionORAddressComponents(PDSParameter):
    pass


physical_delivery_personal_name = univ.Integer(13)


class PhysicalDeliveryPersonalName(PDSParameter):
    pass


physical_delivery_organization_name = univ.Integer(14)


class PhysicalDeliveryOrganizationName(PDSParameter):
    pass


extension_physical_delivery_address_components = univ.Integer(15)


class ExtensionPhysicalDeliveryAddressComponents(PDSParameter):
    pass


unformatted_postal_address = univ.Integer(16)

postal_code = univ.Integer(9)


class PostalCode(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "numeric-code",
            char.NumericString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_postal_code_length)
            ),
        ),
        namedtype.NamedType(
            "printable-code",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_postal_code_length)
            ),
        ),
    )


class PhysicalDeliveryCountryName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "x121-dcc-code",
            char.NumericString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    ub_country_name_numeric_length, ub_country_name_numeric_length
                )
            ),
        ),
        namedtype.NamedType(
            "iso-3166-alpha2-code",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    ub_country_name_alpha_length, ub_country_name_alpha_length
                )
            ),
        ),
    )


class PDSName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_pds_name_length
    )


physical_delivery_country_name = univ.Integer(8)


class TeletexOrganizationalUnitName(char.TeletexString):
    subtypeSpec = char.TeletexString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_organizational_unit_name_length
    )


pds_name = univ.Integer(7)

teletex_organizational_unit_names = univ.Integer(5)


class TeletexOrganizationalUnitNames(univ.SequenceOf):
    componentType = TeletexOrganizationalUnitName()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(
        1, ub_organizational_units
    )


teletex_personal_name = univ.Integer(4)


class TeletexPersonalName(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "surname",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_surname_length),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0),
            ),
        ),
        namedtype.OptionalNamedType(
            "given-name",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_given_name_length),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1),
            ),
        ),
        namedtype.OptionalNamedType(
            "initials",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_initials_length),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2),
            ),
        ),
        namedtype.OptionalNamedType(
            "generation-qualifier",
            char.TeletexString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_generation_qualifier_length
                ),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3),
            ),
        ),
    )


teletex_organization_name = univ.Integer(3)


class TeletexOrganizationName(char.TeletexString):
    subtypeSpec = char.TeletexString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_organization_name_length
    )


teletex_common_name = univ.Integer(2)


class TeletexCommonName(char.TeletexString):
    subtypeSpec = char.TeletexString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_common_name_length
    )


class CommonName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_common_name_length
    )


common_name = univ.Integer(1)


class ExtensionAttribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "extension-attribute-type",
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(0, ub_extension_attributes),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0),
            ),
        ),
        namedtype.NamedType(
            "extension-attribute-value",
            univ.Any().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


class ExtensionAttributes(univ.SetOf):
    componentType = ExtensionAttribute()
    sizeSpec = univ.SetOf.sizeSpec + constraint.ValueSizeConstraint(
        1, ub_extension_attributes
    )


class BuiltInDomainDefinedAttribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "type",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_domain_defined_attribute_type_length
                )
            ),
        ),
        namedtype.NamedType(
            "value",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_domain_defined_attribute_value_length
                )
            ),
        ),
    )


class BuiltInDomainDefinedAttributes(univ.SequenceOf):
    componentType = BuiltInDomainDefinedAttribute()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(
        1, ub_domain_defined_attributes
    )


class OrganizationalUnitName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_organizational_unit_name_length
    )


class OrganizationalUnitNames(univ.SequenceOf):
    componentType = OrganizationalUnitName()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(
        1, ub_organizational_units
    )


class PersonalName(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "surname",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_surname_length),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0),
            ),
        ),
        namedtype.OptionalNamedType(
            "given-name",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_given_name_length),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1),
            ),
        ),
        namedtype.OptionalNamedType(
            "initials",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_initials_length),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2),
            ),
        ),
        namedtype.OptionalNamedType(
            "generation-qualifier",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    1, ub_generation_qualifier_length
                ),
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3),
            ),
        ),
    )


class NumericUserIdentifier(char.NumericString):
    subtypeSpec = char.NumericString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_numeric_user_id_length
    )


class OrganizationName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_organization_name_length
    )


class PrivateDomainName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "numeric",
            char.NumericString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_name_length)
            ),
        ),
        namedtype.NamedType(
            "printable",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_name_length)
            ),
        ),
    )


class TerminalIdentifier(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_terminal_id_length
    )


class X121Address(char.NumericString):
    subtypeSpec = char.NumericString.subtypeSpec + constraint.ValueSizeConstraint(
        1, ub_x121_address_length
    )


class NetworkAddress(X121Address):
    pass


class AdministrationDomainName(univ.Choice):
    tagSet = univ.Choice.tagSet.tagExplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 2)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "numeric",
            char.NumericString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(0, ub_domain_name_length)
            ),
        ),
        namedtype.NamedType(
            "printable",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(0, ub_domain_name_length)
            ),
        ),
    )


class CountryName(univ.Choice):
    tagSet = univ.Choice.tagSet.tagExplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 1)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "x121-dcc-code",
            char.NumericString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    ub_country_name_numeric_length, ub_country_name_numeric_length
                )
            ),
        ),
        namedtype.NamedType(
            "iso-3166-alpha2-code",
            char.PrintableString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(
                    ub_country_name_alpha_length, ub_country_name_alpha_length
                )
            ),
        ),
    )


class BuiltInStandardAttributes(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType("country-name", CountryName()),
        namedtype.OptionalNamedType(
            "administration-domain-name", AdministrationDomainName()
        ),
        namedtype.OptionalNamedType(
            "network-address",
            NetworkAddress().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "terminal-identifier",
            TerminalIdentifier().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "private-domain-name",
            PrivateDomainName().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "organization-name",
            OrganizationName().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
        namedtype.OptionalNamedType(
            "numeric-user-identifier",
            NumericUserIdentifier().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
            ),
        ),
        namedtype.OptionalNamedType(
            "personal-name",
            PersonalName().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
            ),
        ),
        namedtype.OptionalNamedType(
            "organizational-unit-names",
            OrganizationalUnitNames().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 6)
            ),
        ),
    )


class ORAddress(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "built-in-standard-attributes", BuiltInStandardAttributes()
        ),
        namedtype.OptionalNamedType(
            "built-in-domain-defined-attributes", BuiltInDomainDefinedAttributes()
        ),
        namedtype.OptionalNamedType("extension-attributes", ExtensionAttributes()),
    )


#
# PKIX1Implicit88
#

id_ce_invalidityDate = univ.ObjectIdentifier("2.5.29.24")


class InvalidityDate(useful.GeneralizedTime):
    pass


id_holdinstruction_none = univ.ObjectIdentifier("2.2.840.10040.2.1")
id_holdinstruction_callissuer = univ.ObjectIdentifier("2.2.840.10040.2.2")
id_holdinstruction_reject = univ.ObjectIdentifier("2.2.840.10040.2.3")

holdInstruction = univ.ObjectIdentifier("2.2.840.10040.2")

id_ce_holdInstructionCode = univ.ObjectIdentifier("2.5.29.23")


class HoldInstructionCode(univ.ObjectIdentifier):
    pass


id_ce_cRLReasons = univ.ObjectIdentifier("2.5.29.21")


class CRLReason(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ("unspecified", 0),
        ("keyCompromise", 1),
        ("cACompromise", 2),
        ("affiliationChanged", 3),
        ("superseded", 4),
        ("cessationOfOperation", 5),
        ("certificateHold", 6),
        ("removeFromCRL", 8),
    )


id_ce_cRLNumber = univ.ObjectIdentifier("2.5.29.20")


class CRLNumber(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueSizeConstraint(0, MAX)


class BaseCRLNumber(CRLNumber):
    pass


id_kp_serverAuth = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.1")
id_kp_clientAuth = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.2")
id_kp_codeSigning = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.3")
id_kp_emailProtection = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.4")
id_kp_ipsecEndSystem = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.5")
id_kp_ipsecTunnel = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.6")
id_kp_ipsecUser = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.7")
id_kp_timeStamping = univ.ObjectIdentifier("1.3.6.1.5.5.7.3.8")
id_pe_authorityInfoAccess = univ.ObjectIdentifier("1.3.6.1.5.5.7.1.1")
id_ce_extKeyUsage = univ.ObjectIdentifier("2.5.29.37")


class KeyPurposeId(univ.ObjectIdentifier):
    pass


class ExtKeyUsageSyntax(univ.SequenceOf):
    componentType = KeyPurposeId()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class ReasonFlags(univ.BitString):
    namedValues = namedval.NamedValues(
        ("unused", 0),
        ("keyCompromise", 1),
        ("cACompromise", 2),
        ("affiliationChanged", 3),
        ("superseded", 4),
        ("cessationOfOperation", 5),
        ("certificateHold", 6),
    )


class SkipCerts(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueSizeConstraint(0, MAX)


id_ce_policyConstraints = univ.ObjectIdentifier("2.5.29.36")


class PolicyConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "requireExplicitPolicy",
            SkipCerts().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "inhibitPolicyMapping",
            SkipCerts().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
    )


id_ce_basicConstraints = univ.ObjectIdentifier("2.5.29.19")


class BasicConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType("cA", univ.Boolean(False)),
        namedtype.OptionalNamedType(
            "pathLenConstraint",
            univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)),
        ),
    )


id_ce_subjectDirectoryAttributes = univ.ObjectIdentifier("2.5.29.9")


class EDIPartyName(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "nameAssigner",
            DirectoryString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "partyName",
            DirectoryString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


id_ce_deltaCRLIndicator = univ.ObjectIdentifier("2.5.29.27")


class BaseDistance(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueRangeConstraint(0, MAX)


id_ce_cRLDistributionPoints = univ.ObjectIdentifier("2.5.29.31")


id_ce_issuingDistributionPoint = univ.ObjectIdentifier("2.5.29.28")


id_ce_nameConstraints = univ.ObjectIdentifier("2.5.29.30")


class DisplayText(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "visibleString",
            char.VisibleString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200)
            ),
        ),
        namedtype.NamedType(
            "bmpString",
            char.BMPString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200)
            ),
        ),
        namedtype.NamedType(
            "utf8String",
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200)
            ),
        ),
    )


class NoticeReference(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("organization", DisplayText()),
        namedtype.NamedType(
            "noticeNumbers", univ.SequenceOf(componentType=univ.Integer())
        ),
    )


class UserNotice(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType("noticeRef", NoticeReference()),
        namedtype.OptionalNamedType("explicitText", DisplayText()),
    )


class CPSuri(char.IA5String):
    pass


class PolicyQualifierId(univ.ObjectIdentifier):
    subtypeSpec = univ.ObjectIdentifier.subtypeSpec + constraint.SingleValueConstraint(
        id_qt_cps, id_qt_unotice
    )


class CertPolicyId(univ.ObjectIdentifier):
    pass


class PolicyQualifierInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("policyQualifierId", PolicyQualifierId()),
        namedtype.NamedType("qualifier", univ.Any()),
    )


id_ce_certificatePolicies = univ.ObjectIdentifier("2.5.29.32")


class PolicyInformation(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("policyIdentifier", CertPolicyId()),
        namedtype.OptionalNamedType(
            "policyQualifiers",
            univ.SequenceOf(componentType=PolicyQualifierInfo()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)
            ),
        ),
    )


class CertificatePolicies(univ.SequenceOf):
    componentType = PolicyInformation()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


id_ce_policyMappings = univ.ObjectIdentifier("2.5.29.33")


class PolicyMapping(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("issuerDomainPolicy", CertPolicyId()),
        namedtype.NamedType("subjectDomainPolicy", CertPolicyId()),
    )


class PolicyMappings(univ.SequenceOf):
    componentType = PolicyMapping()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


id_ce_privateKeyUsagePeriod = univ.ObjectIdentifier("2.5.29.16")


class PrivateKeyUsagePeriod(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "notBefore",
            useful.GeneralizedTime().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "notAfter",
            useful.GeneralizedTime().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
    )


id_ce_keyUsage = univ.ObjectIdentifier("2.5.29.15")


class KeyUsage(univ.BitString):
    namedValues = namedval.NamedValues(
        ("digitalSignature", 0),
        ("nonRepudiation", 1),
        ("keyEncipherment", 2),
        ("dataEncipherment", 3),
        ("keyAgreement", 4),
        ("keyCertSign", 5),
        ("cRLSign", 6),
        ("encipherOnly", 7),
        ("decipherOnly", 8),
    )


id_ce = univ.ObjectIdentifier("2.5.29")

id_ce_authorityKeyIdentifier = univ.ObjectIdentifier("2.5.29.35")


class KeyIdentifier(univ.OctetString):
    pass


id_ce_subjectKeyIdentifier = univ.ObjectIdentifier("2.5.29.14")


class SubjectKeyIdentifier(KeyIdentifier):
    pass


id_ce_certificateIssuer = univ.ObjectIdentifier("2.5.29.29")


id_ce_subjectAltName = univ.ObjectIdentifier("2.5.29.17")


id_ce_issuerAltName = univ.ObjectIdentifier("2.5.29.18")


class AttributeValue(univ.Any):
    pass


class AttributeType(univ.ObjectIdentifier):
    pass


certificateAttributesMap = {}


class AttributeTypeAndValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeType()),
        namedtype.NamedType(
            "value",
            AttributeValue(),
            openType=opentype.OpenType("type", certificateAttributesMap),
        ),
    )


class Attribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type", AttributeType()),
        namedtype.NamedType("vals", univ.SetOf(componentType=AttributeValue())),
    )


class SubjectDirectoryAttributes(univ.SequenceOf):
    componentType = Attribute()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class RelativeDistinguishedName(univ.SetOf):
    componentType = AttributeTypeAndValue()


class RDNSequence(univ.SequenceOf):
    componentType = RelativeDistinguishedName()


class Name(univ.Choice):
    componentType = namedtype.NamedTypes(namedtype.NamedType("", RDNSequence()))


class CertificateSerialNumber(univ.Integer):
    pass


class AnotherName(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("type-id", univ.ObjectIdentifier()),
        namedtype.NamedType(
            "value",
            univ.Any().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
    )


class GeneralName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "otherName",
            AnotherName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType(
            "rfc822Name",
            char.IA5String().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "dNSName",
            char.IA5String().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.NamedType(
            "x400Address",
            ORAddress().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
        namedtype.NamedType(
            "directoryName",
            Name().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
            ),
        ),
        namedtype.NamedType(
            "ediPartyName",
            EDIPartyName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5)
            ),
        ),
        namedtype.NamedType(
            "uniformResourceIdentifier",
            char.IA5String().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 6)
            ),
        ),
        namedtype.NamedType(
            "iPAddress",
            univ.OctetString().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7)
            ),
        ),
        namedtype.NamedType(
            "registeredID",
            univ.ObjectIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 8)
            ),
        ),
    )


class GeneralNames(univ.SequenceOf):
    componentType = GeneralName()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class AccessDescription(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("accessMethod", univ.ObjectIdentifier()),
        namedtype.NamedType("accessLocation", GeneralName()),
    )


class AuthorityInfoAccessSyntax(univ.SequenceOf):
    componentType = AccessDescription()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class AuthorityKeyIdentifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "keyIdentifier",
            KeyIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "authorityCertIssuer",
            GeneralNames().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "authorityCertSerialNumber",
            CertificateSerialNumber().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
    )


class DistributionPointName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType(
            "fullName",
            GeneralNames().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType(
            "nameRelativeToCRLIssuer",
            RelativeDistinguishedName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
    )


class DistributionPoint(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "distributionPoint",
            DistributionPointName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "reasons",
            ReasonFlags().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "cRLIssuer",
            GeneralNames().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)
            ),
        ),
    )


class CRLDistPointsSyntax(univ.SequenceOf):
    componentType = DistributionPoint()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class IssuingDistributionPoint(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "distributionPoint",
            DistributionPointName().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.NamedType(
            "onlyContainsUserCerts",
            univ.Boolean(False).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.NamedType(
            "onlyContainsCACerts",
            univ.Boolean(False).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "onlySomeReasons",
            ReasonFlags().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
        namedtype.NamedType(
            "indirectCRL",
            univ.Boolean(False).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
            ),
        ),
    )


class GeneralSubtree(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("base", GeneralName()),
        namedtype.DefaultedNamedType(
            "minimum",
            BaseDistance(0).subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "maximum",
            BaseDistance().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
    )


class GeneralSubtrees(univ.SequenceOf):
    componentType = GeneralSubtree()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class NameConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType(
            "permittedSubtrees",
            GeneralSubtrees().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
        namedtype.OptionalNamedType(
            "excludedSubtrees",
            GeneralSubtrees().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)
            ),
        ),
    )


class CertificateIssuer(GeneralNames):
    pass


class SubjectAltName(GeneralNames):
    pass


class IssuerAltName(GeneralNames):
    pass


certificateExtensionsMap = {}


class Extension(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("extnID", univ.ObjectIdentifier()),
        namedtype.DefaultedNamedType("critical", univ.Boolean("False")),
        namedtype.NamedType(
            "extnValue",
            univ.OctetString(),
            openType=opentype.OpenType("extnID", certificateExtensionsMap),
        ),
    )


class Extensions(univ.SequenceOf):
    componentType = Extension()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class SubjectPublicKeyInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("algorithm", AlgorithmIdentifier()),
        namedtype.NamedType("subjectPublicKey", univ.BitString()),
    )


class UniqueIdentifier(univ.BitString):
    pass


class Time(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("utcTime", useful.UTCTime()),
        namedtype.NamedType("generalTime", useful.GeneralizedTime()),
    )


class Validity(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("notBefore", Time()),
        namedtype.NamedType("notAfter", Time()),
    )


class Version(univ.Integer):
    namedValues = namedval.NamedValues(("v1", 0), ("v2", 1), ("v3", 2))


class TBSCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType(
            "version",
            Version("v1").subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)
            ),
        ),
        namedtype.NamedType("serialNumber", CertificateSerialNumber()),
        namedtype.NamedType("signature", AlgorithmIdentifier()),
        namedtype.NamedType("issuer", Name()),
        namedtype.NamedType("validity", Validity()),
        namedtype.NamedType("subject", Name()),
        namedtype.NamedType("subjectPublicKeyInfo", SubjectPublicKeyInfo()),
        namedtype.OptionalNamedType(
            "issuerUniqueID",
            UniqueIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)
            ),
        ),
        namedtype.OptionalNamedType(
            "subjectUniqueID",
            UniqueIdentifier().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)
            ),
        ),
        namedtype.OptionalNamedType(
            "extensions",
            Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)
            ),
        ),
    )


class Certificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("tbsCertificate", TBSCertificate()),
        namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("signatureValue", univ.BitString()),
    )


# CRL structures


class RevokedCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("userCertificate", CertificateSerialNumber()),
        namedtype.NamedType("revocationDate", Time()),
        namedtype.OptionalNamedType("crlEntryExtensions", Extensions()),
    )


class TBSCertList(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType("version", Version()),
        namedtype.NamedType("signature", AlgorithmIdentifier()),
        namedtype.NamedType("issuer", Name()),
        namedtype.NamedType("thisUpdate", Time()),
        namedtype.OptionalNamedType("nextUpdate", Time()),
        namedtype.OptionalNamedType(
            "revokedCertificates", univ.SequenceOf(componentType=RevokedCertificate())
        ),
        namedtype.OptionalNamedType(
            "crlExtensions",
            Extensions().subtype(
                explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )


class CertificateList(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("tbsCertList", TBSCertList()),
        namedtype.NamedType("signatureAlgorithm", AlgorithmIdentifier()),
        namedtype.NamedType("signature", univ.BitString()),
    )


# map of AttributeType -> AttributeValue

_certificateAttributesMapUpdate = {
    id_at_name: X520name(),
    id_at_surname: X520name(),
    id_at_givenName: X520name(),
    id_at_initials: X520name(),
    id_at_generationQualifier: X520name(),
    id_at_commonName: X520CommonName(),
    id_at_localityName: X520LocalityName(),
    id_at_stateOrProvinceName: X520StateOrProvinceName(),
    id_at_organizationName: X520OrganizationName(),
    id_at_organizationalUnitName: X520OrganizationalUnitName(),
    id_at_title: X520Title(),
    id_at_dnQualifier: X520dnQualifier(),
    id_at_countryName: X520countryName(),
    emailAddress: Pkcs9email(),
}

certificateAttributesMap.update(_certificateAttributesMapUpdate)


# map of Certificate Extension OIDs to Extensions

_certificateExtensionsMapUpdate = {
    id_ce_authorityKeyIdentifier: AuthorityKeyIdentifier(),
    id_ce_subjectKeyIdentifier: SubjectKeyIdentifier(),
    id_ce_keyUsage: KeyUsage(),
    id_ce_privateKeyUsagePeriod: PrivateKeyUsagePeriod(),
    id_ce_certificatePolicies: CertificatePolicies(),
    id_ce_policyMappings: PolicyMappings(),
    id_ce_subjectAltName: SubjectAltName(),
    id_ce_issuerAltName: IssuerAltName(),
    id_ce_subjectDirectoryAttributes: SubjectDirectoryAttributes(),
    id_ce_basicConstraints: BasicConstraints(),
    id_ce_nameConstraints: NameConstraints(),
    id_ce_policyConstraints: PolicyConstraints(),
    id_ce_extKeyUsage: ExtKeyUsageSyntax(),
    id_ce_cRLDistributionPoints: CRLDistPointsSyntax(),
    id_pe_authorityInfoAccess: AuthorityInfoAccessSyntax(),
    id_ce_cRLNumber: univ.Integer(),
    id_ce_deltaCRLIndicator: BaseCRLNumber(),
    id_ce_issuingDistributionPoint: IssuingDistributionPoint(),
    id_ce_cRLReasons: CRLReason(),
    id_ce_holdInstructionCode: univ.ObjectIdentifier(),
    id_ce_invalidityDate: useful.GeneralizedTime(),
    id_ce_certificateIssuer: GeneralNames(),
}

certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Other Certificates Extension
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5697.txt

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc4055, rfc5280

# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

CertificateSerialNumber = rfc5280.CertificateSerialNumber

GeneralNames = rfc5280.GeneralNames


# Imports from RFC 4055

id_sha1 = rfc4055.id_sha1


# Imports from RFC 5055
# These are defined here because a module for RFC 5055 does not exist yet


class SCVPIssuerSerial(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("issuer", GeneralNames()),
        namedtype.NamedType("serialNumber", CertificateSerialNumber()),
    )


sha1_alg_id = AlgorithmIdentifier()
sha1_alg_id["algorithm"] = id_sha1


class SCVPCertID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certHash", univ.OctetString()),
        namedtype.NamedType("issuerSerial", SCVPIssuerSerial()),
        namedtype.DefaultedNamedType("hashAlgorithm", sha1_alg_id),
    )


# Other Certificates Extension

id_pe_otherCerts = univ.ObjectIdentifier(
    (
        1,
        3,
        6,
        1,
        5,
        5,
        7,
        1,
        19,
    )
)


class OtherCertificates(univ.SequenceOf):
    componentType = SCVPCertID()


# Update of certificate extension map in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_pe_otherCerts: OtherCertificates(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#!/usr/bin/env python
#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
import hashlib
import sys

try:
    import urllib2

except ImportError:
    import urllib.request as urllib2

from pyasn1.codec.der import decoder, encoder
from pyasn1.type import univ

from pyasn1_modules import pem, rfc2459, rfc2560

sha1oid = univ.ObjectIdentifier((1, 3, 14, 3, 2, 26))


# noinspection PyClassHasNoInit
class ValueOnlyBitStringEncoder(encoder.encoder.BitStringEncoder):
    # These methods just do not encode tag and length fields of TLV
    def encodeTag(self, *args):
        return ""

    def encodeLength(self, *args):
        return ""

    def encodeValue(*args):
        substrate, isConstructed = encoder.encoder.BitStringEncoder.encodeValue(*args)
        # OCSP-specific hack follows: cut off the "unused bit count"
        # encoded bit-string value.
        return substrate[1:], isConstructed

    def __call__(self, bitStringValue):
        return self.encode(None, bitStringValue, defMode=True, maxChunkSize=0)


valueOnlyBitStringEncoder = ValueOnlyBitStringEncoder()


# noinspection PyShadowingNames
def mkOcspRequest(issuerCert, userCert):
    issuerTbsCertificate = issuerCert.getComponentByName("tbsCertificate")
    issuerSubject = issuerTbsCertificate.getComponentByName("subject")

    userTbsCertificate = userCert.getComponentByName("tbsCertificate")
    userIssuer = userTbsCertificate.getComponentByName("issuer")

    assert issuerSubject == userIssuer, "{}\n{}".format(
        issuerSubject.prettyPrint(),
        userIssuer.prettyPrint(),
    )

    userIssuerHash = hashlib.sha1(encoder.encode(userIssuer)).digest()

    issuerSubjectPublicKey = issuerTbsCertificate.getComponentByName(
        "subjectPublicKeyInfo"
    ).getComponentByName("subjectPublicKey")

    issuerKeyHash = hashlib.sha1(
        valueOnlyBitStringEncoder(issuerSubjectPublicKey)
    ).digest()

    userSerialNumber = userTbsCertificate.getComponentByName("serialNumber")

    # Build request object

    request = rfc2560.Request()

    reqCert = request.setComponentByName("reqCert").getComponentByName("reqCert")

    hashAlgorithm = reqCert.setComponentByName("hashAlgorithm").getComponentByName(
        "hashAlgorithm"
    )
    hashAlgorithm.setComponentByName("algorithm", sha1oid)

    reqCert.setComponentByName("issuerNameHash", userIssuerHash)
    reqCert.setComponentByName("issuerKeyHash", issuerKeyHash)
    reqCert.setComponentByName("serialNumber", userSerialNumber)

    ocspRequest = rfc2560.OCSPRequest()

    tbsRequest = ocspRequest.setComponentByName("tbsRequest").getComponentByName(
        "tbsRequest"
    )
    tbsRequest.setComponentByName("version", "v1")

    requestList = tbsRequest.setComponentByName("requestList").getComponentByName(
        "requestList"
    )
    requestList.setComponentByPosition(0, request)

    return ocspRequest


def parseOcspResponse(ocspResponse):
    responseStatus = ocspResponse.getComponentByName("responseStatus")
    assert responseStatus == rfc2560.OCSPResponseStatus(
        "successful"
    ), responseStatus.prettyPrint()
    responseBytes = ocspResponse.getComponentByName("responseBytes")
    responseType = responseBytes.getComponentByName("responseType")
    assert responseType == rfc2560.id_pkix_ocsp_basic, responseType.prettyPrint()

    response = responseBytes.getComponentByName("response")

    basicOCSPResponse, _ = decoder.decode(
        response, asn1Spec=rfc2560.BasicOCSPResponse()
    )

    tbsResponseData = basicOCSPResponse.getComponentByName("tbsResponseData")

    response0 = tbsResponseData.getComponentByName("responses").getComponentByPosition(
        0
    )

    return (
        tbsResponseData.getComponentByName("producedAt"),
        response0.getComponentByName("certID"),
        response0.getComponentByName("certStatus").getName(),
        response0.getComponentByName("thisUpdate"),
    )


if len(sys.argv) != 2:
    print(
        """Usage:
$ cat CACertificate.pem userCertificate.pem | %s <ocsp-responder-url>"""
        % sys.argv[0]
    )
    sys.exit(-1)
else:
    ocspUrl = sys.argv[1]

# Parse CA and user certificates

issuerCert, _ = decoder.decode(
    pem.readPemBlocksFromFile(
        sys.stdin, ("-----BEGIN CERTIFICATE-----", "-----END CERTIFICATE-----")
    )[1],
    asn1Spec=rfc2459.Certificate(),
)
# noinspection PyRedeclaration
userCert, _ = decoder.decode(
    pem.readPemBlocksFromFile(
        sys.stdin, ("-----BEGIN CERTIFICATE-----", "-----END CERTIFICATE-----")
    )[1],
    asn1Spec=rfc2459.Certificate(),
)

# Build OCSP request

ocspReq = mkOcspRequest(issuerCert, userCert)

# Use HTTP POST to get response (see Appendix A of RFC 2560)
# In case you need proxies, set the http_proxy env variable

httpReq = urllib2.Request(
    ocspUrl, encoder.encode(ocspReq), {"Content-Type": "application/ocsp-request"}
)
httpRsp = urllib2.urlopen(httpReq).read()

# Process OCSP response

# noinspection PyRedeclaration
ocspRsp, _ = decoder.decode(httpRsp, asn1Spec=rfc2560.OCSPResponse())

producedAt, certId, certStatus, thisUpdate = parseOcspResponse(ocspRsp)

print(
    "Certificate ID %s is %s at %s till %s\n"
    % (certId.getComponentByName("serialNumber"), certStatus, producedAt, thisUpdate)
)



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Use of the RSA-KEM Key Transport Algorithm in the CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5990.txt
#

from pyasn1.type import constraint, namedtype, univ

from pyasn1_modules import rfc5280

MAX = float("inf")


def _OID(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))
    return univ.ObjectIdentifier(output)


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier


# Useful types and definitions


class NullParms(univ.Null):
    pass


# Object identifier arcs

is18033_2 = _OID(1, 0, 18033, 2)

nistAlgorithm = _OID(2, 16, 840, 1, 101, 3, 4)

pkcs_1 = _OID(1, 2, 840, 113549, 1, 1)

x9_44 = _OID(1, 3, 133, 16, 840, 9, 44)

x9_44_components = _OID(x9_44, 1)


# Types for algorithm identifiers


class Camellia_KeyWrappingScheme(AlgorithmIdentifier):
    pass


class DataEncapsulationMechanism(AlgorithmIdentifier):
    pass


class KDF2_HashFunction(AlgorithmIdentifier):
    pass


class KDF3_HashFunction(AlgorithmIdentifier):
    pass


class KeyDerivationFunction(AlgorithmIdentifier):
    pass


class KeyEncapsulationMechanism(AlgorithmIdentifier):
    pass


class X9_SymmetricKeyWrappingScheme(AlgorithmIdentifier):
    pass


# RSA-KEM Key Transport Algorithm

id_rsa_kem = _OID(1, 2, 840, 113549, 1, 9, 16, 3, 14)


class GenericHybridParameters(univ.Sequence):
    pass


GenericHybridParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("kem", KeyEncapsulationMechanism()),
    namedtype.NamedType("dem", DataEncapsulationMechanism()),
)


rsa_kem = AlgorithmIdentifier()
rsa_kem["algorithm"] = id_rsa_kem
rsa_kem["parameters"] = GenericHybridParameters()


# KEM-RSA Key Encapsulation Mechanism

id_kem_rsa = _OID(is18033_2, 2, 4)


class KeyLength(univ.Integer):
    pass


KeyLength.subtypeSpec = constraint.ValueRangeConstraint(1, MAX)


class RsaKemParameters(univ.Sequence):
    pass


RsaKemParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("keyDerivationFunction", KeyDerivationFunction()),
    namedtype.NamedType("keyLength", KeyLength()),
)


kem_rsa = AlgorithmIdentifier()
kem_rsa["algorithm"] = id_kem_rsa
kem_rsa["parameters"] = RsaKemParameters()


# Key Derivation Functions

id_kdf_kdf2 = _OID(x9_44_components, 1)

id_kdf_kdf3 = _OID(x9_44_components, 2)


kdf2 = AlgorithmIdentifier()
kdf2["algorithm"] = id_kdf_kdf2
kdf2["parameters"] = KDF2_HashFunction()

kdf3 = AlgorithmIdentifier()
kdf3["algorithm"] = id_kdf_kdf3
kdf3["parameters"] = KDF3_HashFunction()


# Hash Functions

id_sha1 = _OID(1, 3, 14, 3, 2, 26)

id_sha224 = _OID(2, 16, 840, 1, 101, 3, 4, 2, 4)

id_sha256 = _OID(2, 16, 840, 1, 101, 3, 4, 2, 1)

id_sha384 = _OID(2, 16, 840, 1, 101, 3, 4, 2, 2)

id_sha512 = _OID(2, 16, 840, 1, 101, 3, 4, 2, 3)


sha1 = AlgorithmIdentifier()
sha1["algorithm"] = id_sha1
sha1["parameters"] = univ.Null("")

sha224 = AlgorithmIdentifier()
sha224["algorithm"] = id_sha224
sha224["parameters"] = univ.Null("")

sha256 = AlgorithmIdentifier()
sha256["algorithm"] = id_sha256
sha256["parameters"] = univ.Null("")

sha384 = AlgorithmIdentifier()
sha384["algorithm"] = id_sha384
sha384["parameters"] = univ.Null("")

sha512 = AlgorithmIdentifier()
sha512["algorithm"] = id_sha512
sha512["parameters"] = univ.Null("")


# Symmetric Key-Wrapping Schemes

id_aes128_Wrap = _OID(nistAlgorithm, 1, 5)

id_aes192_Wrap = _OID(nistAlgorithm, 1, 25)

id_aes256_Wrap = _OID(nistAlgorithm, 1, 45)

id_alg_CMS3DESwrap = _OID(1, 2, 840, 113549, 1, 9, 16, 3, 6)

id_camellia128_Wrap = _OID(1, 2, 392, 200011, 61, 1, 1, 3, 2)

id_camellia192_Wrap = _OID(1, 2, 392, 200011, 61, 1, 1, 3, 3)

id_camellia256_Wrap = _OID(1, 2, 392, 200011, 61, 1, 1, 3, 4)


aes128_Wrap = AlgorithmIdentifier()
aes128_Wrap["algorithm"] = id_aes128_Wrap
# aes128_Wrap['parameters'] are absent

aes192_Wrap = AlgorithmIdentifier()
aes192_Wrap["algorithm"] = id_aes128_Wrap
# aes192_Wrap['parameters'] are absent

aes256_Wrap = AlgorithmIdentifier()
aes256_Wrap["algorithm"] = id_sha256
# aes256_Wrap['parameters'] are absent

tdes_Wrap = AlgorithmIdentifier()
tdes_Wrap["algorithm"] = id_alg_CMS3DESwrap
tdes_Wrap["parameters"] = univ.Null("")

camellia128_Wrap = AlgorithmIdentifier()
camellia128_Wrap["algorithm"] = id_camellia128_Wrap
# camellia128_Wrap['parameters'] are absent

camellia192_Wrap = AlgorithmIdentifier()
camellia192_Wrap["algorithm"] = id_camellia192_Wrap
# camellia192_Wrap['parameters'] are absent

camellia256_Wrap = AlgorithmIdentifier()
camellia256_Wrap["algorithm"] = id_camellia256_Wrap
# camellia256_Wrap['parameters'] are absent


# Update the Algorithm Identifier map in rfc5280.py.
# Note that the ones that must not have parameters are not added to the map.

_algorithmIdentifierMapUpdate = {
    id_rsa_kem: GenericHybridParameters(),
    id_kem_rsa: RsaKemParameters(),
    id_kdf_kdf2: KDF2_HashFunction(),
    id_kdf_kdf3: KDF3_HashFunction(),
    id_sha1: univ.Null(),
    id_sha224: univ.Null(),
    id_sha256: univ.Null(),
    id_sha384: univ.Null(),
    id_sha512: univ.Null(),
    id_alg_CMS3DESwrap: univ.Null(),
}

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# PKCS#10 syntax
#
# ASN.1 source from:
# http://tools.ietf.org/html/rfc2314
#
# Sample captures could be obtained with "openssl req" command
#
from pyasn1_modules.rfc2459 import *


class Attributes(univ.SetOf):
    componentType = Attribute()


class Version(univ.Integer):
    pass


class CertificationRequestInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("version", Version()),
        namedtype.NamedType("subject", Name()),
        namedtype.NamedType("subjectPublicKeyInfo", SubjectPublicKeyInfo()),
        namedtype.NamedType(
            "attributes",
            Attributes().subtype(
                implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)
            ),
        ),
    )


class Signature(univ.BitString):
    pass


class SignatureAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class CertificationRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("certificationRequestInfo", CertificationRequestInfo()),
        namedtype.NamedType("signatureAlgorithm", SignatureAlgorithmIdentifier()),
        namedtype.NamedType("signature", Signature()),
    )



#
# This file is part of pyasn1-modules software.
#
# Created by Russ Housley with some assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Authentication Context Certificate Extension
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7773.txt
#

from pyasn1.type import char, constraint, namedtype, univ

from pyasn1_modules import rfc5280

MAX = float("inf")


# Authentication Context Extension

e_legnamnden = univ.ObjectIdentifier("1.2.752.201")

id_eleg_ce = e_legnamnden + (5,)

id_ce_authContext = id_eleg_ce + (1,)


class AuthenticationContext(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType("contextType", char.UTF8String()),
        namedtype.OptionalNamedType("contextInfo", char.UTF8String()),
    )


class AuthenticationContexts(univ.SequenceOf):
    componentType = AuthenticationContext()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


# Map of Certificate Extension OIDs to Extensions added to the
# ones that are in rfc5280.py

_certificateExtensionsMapUpdate = {
    id_ce_authContext: AuthenticationContexts(),
}

rfc5280.certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



# This file is being contributed to pyasn1-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to add maps for opentypes.
#
# Copyright (c) 2019, Vigil Security, LLC
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Elliptic Curve Cryptography Subject Public Key Information
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5480.txt


# What can be imported from rfc4055.py ?

from pyasn1.type import namedtype, univ

from pyasn1_modules import rfc3279, rfc5280

# These structures are the same as RFC 3279.

DHPublicKey = rfc3279.DHPublicKey

DSAPublicKey = rfc3279.DSAPublicKey

ValidationParms = rfc3279.ValidationParms

DomainParameters = rfc3279.DomainParameters

ECDSA_Sig_Value = rfc3279.ECDSA_Sig_Value

ECPoint = rfc3279.ECPoint

KEA_Parms_Id = rfc3279.KEA_Parms_Id

RSAPublicKey = rfc3279.RSAPublicKey


# RFC 5480 changed the names of these structures from RFC 3279.

DSS_Parms = rfc3279.Dss_Parms

DSA_Sig_Value = rfc3279.Dss_Sig_Value


# RFC 3279 defines a more complex alternative for ECParameters.
# RFC 5480 narrows the definition to a single CHOICE: namedCurve.


class ECParameters(univ.Choice):
    pass


ECParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType("namedCurve", univ.ObjectIdentifier())
)


# OIDs for Message Digest Algorithms

id_md2 = univ.ObjectIdentifier("1.2.840.113549.2.2")

id_md5 = univ.ObjectIdentifier("1.2.840.113549.2.5")

id_sha1 = univ.ObjectIdentifier("1.3.14.3.2.26")

id_sha224 = univ.ObjectIdentifier("2.16.840.1.101.3.4.2.4")

id_sha256 = univ.ObjectIdentifier("2.16.840.1.101.3.4.2.1")

id_sha384 = univ.ObjectIdentifier("2.16.840.1.101.3.4.2.2")

id_sha512 = univ.ObjectIdentifier("2.16.840.1.101.3.4.2.3")


# OID for RSA PK Algorithm and Key

rsaEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.1")


# OID for DSA PK Algorithm, Key, and Parameters

id_dsa = univ.ObjectIdentifier("1.2.840.10040.4.1")


# OID for Diffie-Hellman PK Algorithm, Key, and Parameters

dhpublicnumber = univ.ObjectIdentifier("1.2.840.10046.2.1")

# OID for KEA PK Algorithm and Parameters

id_keyExchangeAlgorithm = univ.ObjectIdentifier("2.16.840.1.101.2.1.1.22")


# OIDs for Elliptic Curve Algorithm ID, Key, and Parameters
# Note that ECDSA keys always use this OID

id_ecPublicKey = univ.ObjectIdentifier("1.2.840.10045.2.1")

id_ecDH = univ.ObjectIdentifier("1.3.132.1.12")

id_ecMQV = univ.ObjectIdentifier("1.3.132.1.13")


# OIDs for RSA Signature Algorithms

md2WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.2")

md5WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.4")

sha1WithRSAEncryption = univ.ObjectIdentifier("1.2.840.113549.1.1.5")


# OIDs for DSA Signature Algorithms

id_dsa_with_sha1 = univ.ObjectIdentifier("1.2.840.10040.4.3")

id_dsa_with_sha224 = univ.ObjectIdentifier("2.16.840.1.101.3.4.3.1")

id_dsa_with_sha256 = univ.ObjectIdentifier("2.16.840.1.101.3.4.3.2")


# OIDs for ECDSA Signature Algorithms

ecdsa_with_SHA1 = univ.ObjectIdentifier("1.2.840.10045.4.1")

ecdsa_with_SHA224 = univ.ObjectIdentifier("1.2.840.10045.4.3.1")

ecdsa_with_SHA256 = univ.ObjectIdentifier("1.2.840.10045.4.3.2")

ecdsa_with_SHA384 = univ.ObjectIdentifier("1.2.840.10045.4.3.3")

ecdsa_with_SHA512 = univ.ObjectIdentifier("1.2.840.10045.4.3.4")


# OIDs for Named Elliptic Curves

secp192r1 = univ.ObjectIdentifier("1.2.840.10045.3.1.1")

sect163k1 = univ.ObjectIdentifier("1.3.132.0.1")

sect163r2 = univ.ObjectIdentifier("1.3.132.0.15")

secp224r1 = univ.ObjectIdentifier("1.3.132.0.33")

sect233k1 = univ.ObjectIdentifier("1.3.132.0.26")

sect233r1 = univ.ObjectIdentifier("1.3.132.0.27")

secp256r1 = univ.ObjectIdentifier("1.2.840.10045.3.1.7")

sect283k1 = univ.ObjectIdentifier("1.3.132.0.16")

sect283r1 = univ.ObjectIdentifier("1.3.132.0.17")

secp384r1 = univ.ObjectIdentifier("1.3.132.0.34")

sect409k1 = univ.ObjectIdentifier("1.3.132.0.36")

sect409r1 = univ.ObjectIdentifier("1.3.132.0.37")

secp521r1 = univ.ObjectIdentifier("1.3.132.0.35")

sect571k1 = univ.ObjectIdentifier("1.3.132.0.38")

sect571r1 = univ.ObjectIdentifier("1.3.132.0.39")


# Map of Algorithm Identifier OIDs to Parameters
# The algorithm is not included if the parameters MUST be absent

_algorithmIdentifierMapUpdate = {
    rsaEncryption: univ.Null(),
    md2WithRSAEncryption: univ.Null(),
    md5WithRSAEncryption: univ.Null(),
    sha1WithRSAEncryption: univ.Null(),
    id_dsa: DSS_Parms(),
    dhpublicnumber: DomainParameters(),
    id_keyExchangeAlgorithm: KEA_Parms_Id(),
    id_ecPublicKey: ECParameters(),
    id_ecDH: ECParameters(),
    id_ecMQV: ECParameters(),
}


# Add these Algorithm Identifier map entries to the ones in rfc5280.py

rfc5280.algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#!/usr/bin/env python
#
# This file is part of pyasn1-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pyasn1/license.html
#
# Read unencrypted PKCS#1/PKIX-compliant, PEM&DER encoded private keys on
# stdin, print them pretty and encode back into original wire format.
# Private keys can be generated with "openssl genrsa|gendsa" commands.
#
import sys

from pyasn1.codec.der import decoder, encoder

from pyasn1_modules import pem, rfc2437, rfc2459

if len(sys.argv) != 1:
    print(
        """Usage:
$ cat rsakey.pem | %s"""
        % sys.argv[0]
    )
    sys.exit(-1)

cnt = 0

while True:
    idx, substrate = pem.readPemBlocksFromFile(
        sys.stdin,
        ("-----BEGIN RSA PRIVATE KEY-----", "-----END RSA PRIVATE KEY-----"),
        ("-----BEGIN DSA PRIVATE KEY-----", "-----END DSA PRIVATE KEY-----"),
    )
    if not substrate:
        break

    if idx == 0:
        asn1Spec = rfc2437.RSAPrivateKey()
    elif idx == 1:
        asn1Spec = rfc2459.DSAPrivateKey()
    else:
        break

    key, rest = decoder.decode(substrate, asn1Spec=asn1Spec)

    if rest:
        substrate = substrate[: -len(rest)]

    print(key.prettyPrint())

    assert encoder.encode(key) == substrate, "pkcs8 recode fails"

    cnt += 1

print("*** %s key(s) re/serialized" % cnt)
